<?php

namespace Drupal\liqpay\Plugin\Basket\Payment;

use Drupal\basket\Plugins\Payment\BasketPaymentInterface;
use Drupal\basket\Plugins\Payment\Annotation\BasketPayment;
use Drupal\liqpay\Form\PaymentForm;
use Drupal\liqpay\Controller\LiqpayPages;

/**
 * @BasketPayment(
 *   id = "liqpay",
 *   name = "LiqPay",
 * )
 */
class BasketLiqpay implements BasketPaymentInterface {

  /**
   * @var \Drupal\liqpay\LiqPay|object|null
   */
	protected $liqPay;

  /**
   * @var \Drupal\basket\Basket|object|null
   */
	protected $basket;

	function __construct(){
		$this->liqPay = \Drupal::getContainer()->get('LiqPay');
		$this->basket = \Drupal::getContainer()->get('Basket');
	}

  /**
   * Alter for making adjustments when creating a payment item.
   *
   * @param $form
   * @param $form_state
   *
   * @return void
   */
	public function settingsFormAlter(&$form, $form_state){
		$tid = $form['tid']['#value'];
		$form['settings'] = [
			'#type' => 'details',
			'#title' => $this->liqPay->t('Settings'),
			'#open'	=> TRUE,
			'#parents' => ['basket_payment_liqpay_settings'],
			'#tree' => TRUE,
			'status' => [
				'#type' => 'select',
				'#title' => $this->liqPay->t('Change order status after payment to'),
				'#options' => $this->basket->Term()->getOptions('fin_status'),
				'#empty_option'	=> $this->liqPay->t('Do not change status'),
				'#default_value' => $this->basket->getSettings('payment_settings', $tid.'.status')
			]
		];
		$form['#submit'][] = [$this, 'formSubmit'];
	}
	public function formSubmit($form, $form_state){
		$tid = $form_state->getValue('tid');
		if(!empty($tid)){
			$this->basket->setSettings(
				'payment_settings', 
        $tid,
        $form_state->getValue('basket_payment_liqpay_settings')
			);
		}
	}

  /**
   * Interpretation of the list of settings for pages with payment types.
   *
   * @param $tid
   *
   * @return array
   */
	public function getSettingsInfoList($tid){
		$items = [];
		if(!empty($settings = $this->basket->getSettings('payment_settings', $tid))){
			$value = $this->liqPay->t('Do not change status');
			if(!empty($settings['status']) && !empty($term = $this->basket->Term()->load($settings['status']))){
				$value = $this->basket->Translate()->t($term->name);
			}
			$items[] = [
				'#type' => 'inline_template',
				'#template' => '<b>{{ label }}: </b> {{ value }}',
				'#context' => [
					'label' => $this->liqPay->t('Change order status after payment to'),
					'value'	=> $value
				]
			];
		}
		return $items;
	}

  /**
   * Creation of payment.
   *
   * @param $entity
   * @param $order
   *
   * @return array|null[]
   */
	public function createPayment($entity, $order){
		if(!empty($order->pay_price)){
			$currency = $this->basket->currency()->load($order->pay_currency);
			$payment = $this->liqPay->load([
				'nid' => $entity->id(),
				'create_new' => TRUE,
				'amount' => $order->pay_price,
				'currency' => $currency->iso
			]);
			if(!empty($payment)){
				return [
					'payID' => $payment->id,
					'redirectUrl'	=> NULL
				];
			}
		}
		return [
			'payID' => NULL,
			'redirectUrl'	=> NULL
		];
	}

  /**
   * Alter page redirects for payment.
   * @param $id
   *
   * @return array|void
   */
	public function loadPayment($id){
		$payment = $this->liqPay->load(['id' => $id]);
		if(!empty($payment)){
			$successes = explode('|', $this->liqPay::LIQPAY_SUCCESS_STATUS);
			return [
				'payment' => $payment,
				'isPay' => !empty($payment->status) && in_array($payment->status, $successes)
			];
		}
	}

  /**
   * Alter page redirects for payment.
   *
   * @param $form
   * @param $form_state
   * @param $payment
   *
   * @return void
   */
	public function paymentFormAlter(&$form, $form_state, $payment){
    (new PaymentForm())->basketPaymentFormAlter($form, $form_state, $payment);
	}

  /**
   * Alter processing pages of interaction between the payment system and the site.
   *
   * @param $pageType
   *
   * @return array|void
   */
	public function basketPaymentPages($pageType){
		switch($pageType){
			case'callback':
				return (new LiqpayPages())->pages('api');
    
			case'cancel':
			case'result':
				return (new LiqpayPages())->pages('payment_result');
    
		}
	}

  /**
   * Order update upon successful ordering by payment item settings.
   *
   * @param $pid
   * @param $Order
   *
   * @return void
   */
	public function updateOrderBySettings($pid, $Order){
		$order = $Order->load();
		if(!empty($order) && !empty($settings = $this->basket->getSettings('payment_settings', $pid))){
			if(!empty($settings['status'])){
				$order->fin_status = $settings['status'];
				$Order->replaceOrder($order);
				$Order->save();
			}
		}
	}
}